// Generated by CoffeeScript 1.7.1
(function() {
  var Duration, NANOS_PER_DAY, NANOS_PER_HOUR, NANOS_PER_MICRO, NANOS_PER_MILLI, NANOS_PER_MINUTE, NANOS_PER_SECOND, Q, Stopwatch, newDuration, newStopwatch, timeOperation, timeOperationAsync;

  Q = require('q');

  NANOS_PER_MICRO = 1000;

  NANOS_PER_MILLI = NANOS_PER_MICRO * 1000;

  NANOS_PER_SECOND = NANOS_PER_MILLI * 1000;

  NANOS_PER_MINUTE = NANOS_PER_SECOND * 60;

  NANOS_PER_HOUR = NANOS_PER_MINUTE * 60;

  NANOS_PER_DAY = NANOS_PER_HOUR * 24;

  Duration = (function() {
    function Duration(duration) {
      this.duration = duration;
    }

    Duration.prototype.nanos = function() {
      return this.duration;
    };

    Duration.prototype.micros = function() {
      return this.duration / NANOS_PER_MICRO;
    };

    Duration.prototype.millis = function() {
      return this.duration / NANOS_PER_MILLI;
    };

    Duration.prototype.seconds = function() {
      return this.durations / NANOS_PER_SECOND;
    };

    Duration.prototype.minutes = function() {
      return this.durations / NANOS_PER_MINUTE;
    };

    Duration.prototype.hours = function() {
      return this.durations / NANOS_PER_HOUR;
    };

    Duration.prototype.days = function() {
      return this.durations / NANOS_PER_DAY;
    };

    Duration.prototype.format = function() {
      var micros, millis, minutes, nanos, seconds;
      nanos = this.duration;
      if (nanos >= NANOS_PER_HOUR) {
        minutes = nanos / NANOS_PER_MINUTE;
        return Math.floor(minutes / 60) + " h, " + Math.floor(minutes % 60) + " min";
      }
      if (nanos >= NANOS_PER_MINUTE) {
        seconds = nanos / NANOS_PER_SECOND;
        return Math.floor(seconds / 60) + " min, " + Math.floor(seconds % 60) + " s";
      }
      if (nanos >= NANOS_PER_SECOND) {
        millis = nanos / NANOS_PER_MILLI;
        return Math.floor(millis / 1000) + "." + this.zPad(Math.floor(millis % 1000)) + " s";
      }
      if (nanos >= NANOS_PER_MILLI) {
        micros = nanos / NANOS_PER_MICRO;
        return Math.floor(micros / 1000) + "." + this.zPad(Math.floor(micros % 1000)) + " ms";
      }
      return Math.floor(nanos / 1000) + "." + this.zPad(Math.floor(nanos % 1000)) + " us";
    };

    Duration.prototype.toString = function() {
      return this.format();
    };

    Duration.prototype.zPad = function(value) {
      if (value < 10) {
        return "00" + value;
      }
      if (value < 100) {
        return "0" + value;
      }
      return "" + value;
    };

    return Duration;

  })();

  Stopwatch = (function() {
    var hrtimeDiffToNanos;

    hrtimeDiffToNanos = function(diff) {
      return diff[0] * 1e9 + diff[1];
    };

    function Stopwatch() {
      this.reset();
    }

    Stopwatch.prototype.reset = function() {
      this.accumulator = 0;
      this.lastTime = null;
      return this;
    };

    Stopwatch.prototype.start = function() {
      if (this.lastTime == null) {
        this.lastTime = process.hrtime();
      }
      return this;
    };

    Stopwatch.prototype.stop = function() {
      var diff;
      if (this.lastTime != null) {
        diff = process.hrtime(this.lastTime);
        this.accumulator += hrtimeDiffToNanos(diff);
        this.lastTime = null;
      }
      return this;
    };

    Stopwatch.prototype.duration = function() {
      return newDuration(this.elapsedNanos());
    };

    Stopwatch.prototype.format = function() {
      return this.duration().format();
    };

    Stopwatch.prototype.toString = function() {
      return this.format();
    };

    Stopwatch.prototype.elapsedNanos = function() {
      var diff;
      if (this.lastTime != null) {
        diff = process.hrtime(this.lastTime);
        return this.accumulator + hrtimeDiffToNanos(diff);
      } else {
        return this.accumulator;
      }
    };

    return Stopwatch;

  })();

  timeOperation = function(operation) {
    var watch;
    watch = newStopwatch().start();
    operation();
    return watch.stop().duration();
  };

  timeOperationAsync = function(operation) {
    var deferred, next, watch;
    deferred = Q.defer();
    watch = newStopwatch().start();
    next = function() {
      var duration;
      duration = watch.stop().duration();
      return deferred.resolve(duration);
    };
    operation(next);
    return deferred.promise;
  };

  newStopwatch = function() {
    return new Stopwatch();
  };

  newDuration = function(nanoseconds) {
    return new Duration(nanoseconds);
  };

  module.exports = {
    duration: newDuration,
    stopwatch: newStopwatch,
    time: timeOperation,
    timeAsync: timeOperationAsync
  };

}).call(this);
